package com.sample.project;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.EventListenerList;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.plaf.ComboBoxUI;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.plaf.metal.MetalComboBoxUI;

import com.sun.java.swing.plaf.motif.MotifComboBoxUI;
import com.sun.java.swing.plaf.windows.WindowsComboBoxUI;

public class TDateComboBox 
		extends JComponent 
		implements PropertyChangeListener,  
					ActionListener, ItemListener {
	
	private JComboBox combo;
		
	/**
	 * Constructs a new TDateComboBox, setting its model equal
	 * to the provided model.
	 * 
	 * @param model the model providing the data for the combo box. 
	 */
	public TDateComboBox(ComboBoxModel model) {
		super();
		this.combo = new JComboBox(model);
	}
		
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#updateUI()
	 */
	public void updateUI() {
		ComboBoxUI cui = (ComboBoxUI)UIManager.getUI(this.combo);
		if (cui instanceof MotifComboBoxUI) {
			cui = new MotifDateComboBoxUI();
		} else if (cui instanceof WindowsComboBoxUI) {
			cui = new WindowsDateComboBoxUI();
		}
		this.combo.setUI(cui);
	}


	private class WindowsDateComboBoxUI extends WindowsComboBoxUI {
		protected ComboPopup createPopup() {
			return new DatePopup(this.comboBox);
		}
	}

	private class MotifDateComboBoxUI extends MotifComboBoxUI {
		protected ComboPopup createPopup() {
			return new DatePopup(this.comboBox);
		}
	}
	
	//////////////////////////////////////////////////////////////
	// DatePopup inner class
	//////////////////////////////////////////////////////////////
	private class DatePopup 
			implements ComboPopup, MouseMotionListener, 
					MouseListener, KeyListener, PopupMenuListener {

		private static final int MAX_DAYS_IN_MONTH = 31;

		private static final int HORIZ_SPACING = 12;

		private JComboBox comboBox;
		private Calendar calendar;
		private JPopupMenu monthPopupMenu;
		private JPanel pnlPopupHeader;
		private JPanel pnlDays = null;
		private JLabel lblSelectedMonth;
		private JLabel lblPrevYear;
		private JLabel lblPrevMonth;
		private JLabel lblNextYear;
		private JLabel lblNextMonth;
		private SimpleDateFormat monthFormat = 
			new SimpleDateFormat("MMM yyyy");

		private Color selectedBackground;
		private Color selectedForeground;
		private Color background;
		private Color foreground;

		private Vector weekDayLabelList;
		private Vector monthDayLabelList;

		protected boolean hideNext = false;
		protected boolean mouseInside = false;
		protected JList list = new JList();

		public DatePopup(JComboBox comboBox) {
			this.comboBox = comboBox;
			this.calendar = Calendar.getInstance();
			
			// check Look and Feel
			this.background = UIManager.getColor("ComboBox.background");
			this.foreground = UIManager.getColor("ComboBox.foreground");
			this.selectedBackground = UIManager.getColor("ComboBox.selectionBackground");
			this.selectedForeground = UIManager.getColor("ComboBox.selectionForeground");

		}

		//========================================
		// begin ComboPopup method implementations
		//
		public void show() {
			try {
				// if setSelectedItem() was called with a valid date, adjust the calendar
				this.calendar.setTime(
						((Calendar)this.comboBox.getSelectedItem()).getTime());
			} catch (Exception e) {
				//
			}
			this.updatePopup();
			this.getMonthPopupMenu().show(
					this.comboBox, 0, this.comboBox.getHeight());
		}

		public void hide() {
			this.getMonthPopupMenu().setVisible(false);
		}

		public JList getList() {
			return this.list;
		}

		public MouseListener getMouseListener() {
			return this;
		}

		public MouseMotionListener getMouseMotionListener() {
			return this;
		}

		public KeyListener getKeyListener() {
			return this;
		}

		public boolean isVisible() {
			return getMonthPopupMenu().isVisible();
		}

		public void uninstallingUI() {
			getMonthPopupMenu().removePopupMenuListener(this);
		}

		//
		// end ComboPopup method implementations
		//======================================

		//===================================================================
		// begin Event Listeners
		//

		// MouseListener

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		// something else registered for MousePressed
		public void mouseClicked(MouseEvent e) {
			if (!SwingUtilities.isLeftMouseButton(e))
				return;
			if (!this.comboBox.isEnabled())
				return;

			if (this.comboBox.isEditable()) {
				this.comboBox.getEditor().getEditorComponent().requestFocus();
			} else {
				//this.comboBox.requestFocus();
			}
			togglePopup();
		}

		public void mouseEntered(MouseEvent e) {
			this.mouseInside = true;
		}

		public void mouseExited(MouseEvent e) {
			this.mouseInside = false;
		}

		// MouseMotionListener
		public void mouseDragged(MouseEvent e) {
		}

		public void mouseMoved(MouseEvent e) {
		}

		// KeyListener
		public void keyPressed(KeyEvent e) {
		}

		public void keyTyped(KeyEvent e) {
		}

		public void keyReleased(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_SPACE || e.getKeyCode() == KeyEvent.VK_ENTER) {
				togglePopup();
			}
		}

		/**
		 * Variables hideNext and mouseInside are used to
		 * hide the popupMenu by clicking the mouse in the JComboBox
		 */
		public void popupMenuCanceled(PopupMenuEvent e) {
		}

		public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
			this.hideNext = this.mouseInside;
		}

		public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		}

		//
		// end Event Listeners
		//=================================================================

		//===================================================================
		// begin Utility methods
		//

		private void togglePopup() {
			if (isVisible() || this.hideNext) {
				hide();
			} else {
				show();
			}
			this.hideNext = false;
		}

		//
		// end Utility methods
		//=================================================================

		private JPopupMenu getMonthPopupMenu() {
			if (this.monthPopupMenu == null) {
				this.monthPopupMenu = new JPopupMenu();
				this.monthPopupMenu.setBorder(BorderFactory.createLineBorder(Color.black));
				this.monthPopupMenu.setLayout(new BorderLayout());
				this.monthPopupMenu.setBackground(this.background);
				this.monthPopupMenu.addPopupMenuListener(this);
				this.monthPopupMenu.add(BorderLayout.NORTH, getPopupHeaderPanel());
				this.monthPopupMenu.add(BorderLayout.CENTER, getDaysPanel());
			}
			return this.monthPopupMenu;
		}

		private JPanel getPopupHeaderPanel() {
			if (this.pnlPopupHeader == null) {
				this.pnlPopupHeader = new JPanel();
				this.pnlPopupHeader.setLayout(new BoxLayout(this.pnlPopupHeader, BoxLayout.X_AXIS));
				this.pnlPopupHeader.setBackground(this.background);
				this.pnlPopupHeader.setOpaque(true);

				this.pnlPopupHeader.add(Box.createHorizontalStrut(HORIZ_SPACING));
				this.pnlPopupHeader.add(getPrevYearLabel());
				this.pnlPopupHeader.add(Box.createHorizontalStrut(HORIZ_SPACING));

				this.pnlPopupHeader.add(getPrevMonthLabel());

				this.pnlPopupHeader.add(Box.createHorizontalGlue());
				this.pnlPopupHeader.add(getSelectedMonthLabel());
				this.pnlPopupHeader.add(Box.createHorizontalGlue());

				this.pnlPopupHeader.add(getNextMonthLabel());

				this.pnlPopupHeader.add(Box.createHorizontalStrut(HORIZ_SPACING));
				this.pnlPopupHeader.add(getNextYearLabel());
				this.pnlPopupHeader.add(Box.createHorizontalStrut(HORIZ_SPACING));
			}

			return this.pnlPopupHeader;
		}

		private JLabel getPrevYearLabel() {
			if (this.lblPrevYear == null) {
				this.lblPrevYear = createUpdateButton(Calendar.YEAR, -1);
				this.lblPrevYear.setText("<<");
				this.lblPrevYear.setToolTipText("Previous Year");
			}
			return this.lblPrevYear;
		}

		private JLabel getPrevMonthLabel() {
			if (this.lblPrevMonth == null) {
				this.lblPrevMonth = createUpdateButton(Calendar.MONTH, -1);
				//                if(getPrevImage() != null )
				//                    _lblPrevMonth.setIcon( getPrevImage());
				//                else
				this.lblPrevMonth.setText("<");

				this.lblPrevMonth.setToolTipText("Previous Month");
			}

			return this.lblPrevMonth;
		}

		private JLabel getNextYearLabel() {
			if (this.lblNextYear == null) {
				this.lblNextYear = createUpdateButton(Calendar.YEAR, 1);
				this.lblNextYear.setText(">>");
				this.lblNextYear.setToolTipText("Next Year");
			}
			return this.lblNextYear;
		}

		private JLabel getNextMonthLabel() {
			if (this.lblNextMonth == null) {
				this.lblNextMonth = createUpdateButton(Calendar.MONTH, 1);
				this.lblNextMonth.setText(">");
				this.lblNextMonth.setToolTipText("Next Month");
			}
			return this.lblNextMonth;
		}

		// Note *** did not use JButton because Popup closes when pressed
		private JLabel createUpdateButton(final int field, final int amount) {
			final JLabel label = new JLabel();
			final Border selectedBorder = new EtchedBorder();
			final Border unselectedBorder = 
				new EmptyBorder(selectedBorder.getBorderInsets(new JLabel()));
			
			label.setBorder(unselectedBorder);
			label.setForeground(this.foreground);
			label.addMouseListener(new MouseAdapter() {
				public void mouseReleased(MouseEvent e) {
					DatePopup.this.calendar.add(field, amount);
					DatePopup.this.updatePopup();
				}

				public void mouseEntered(MouseEvent e) {
					label.setBorder(selectedBorder);
				}

				public void mouseExited(MouseEvent e) {
					label.setBorder(unselectedBorder);
				}
			});
			return label;
		}

		private JLabel getSelectedMonthLabel() {
			if (this.lblSelectedMonth == null) {
				this.lblSelectedMonth = new JLabel("", JLabel.CENTER);
				this.lblSelectedMonth.setForeground(this.foreground);
			}
			return this.lblSelectedMonth;
		}

		private JPanel getDaysPanel() {
			if (this.pnlDays == null) {
				updateDaysPanel();
			}

			return this.pnlDays;
		}

		private void updateDaysPanel() {
			this.pnlDays = new JPanel(new GridLayout(0, 7));
			this.pnlDays.setBackground(this.background);
			this.pnlDays.setOpaque(true);
			for (int i = 0; i < 7; i++) {
				this.pnlDays.add(getWeekDayLabel(i));
			}
			Calendar setupCalendar = (Calendar) this.calendar.clone();
			setupCalendar.set(Calendar.DAY_OF_MONTH, 1);
			int first = setupCalendar.get(Calendar.DAY_OF_WEEK);
			for (int i = 0; i < (first - 1); i++) {
				this.pnlDays.add(new JLabel(""));
			}
			for (int i = 0; i < setupCalendar.getActualMaximum(Calendar.DAY_OF_MONTH); i++) {
				this.pnlDays.add(getMonthDayLabel(i));
			}
		}

		private JLabel getWeekDayLabel(int day) {
			if (this.weekDayLabelList == null) {
				this.weekDayLabelList = new Vector(7);
			}

			while (this.weekDayLabelList.size() < day + 1) {

				int dayInt = this.weekDayLabelList.size() + 1;
				JLabel weekDayLabel = new JLabel();
				weekDayLabel.setHorizontalAlignment(JLabel.CENTER);
				weekDayLabel.setForeground(this.foreground);

				if (dayInt == Calendar.SUNDAY) {
					weekDayLabel.setText("Sun");
				} else if (dayInt == Calendar.MONDAY) {
					weekDayLabel.setText("Mon");
				} else if (dayInt == Calendar.TUESDAY) {
					weekDayLabel.setText("Tue");
				} else if (dayInt == Calendar.WEDNESDAY) {
					weekDayLabel.setText("Wed");
				} else if (dayInt == Calendar.THURSDAY) {
					weekDayLabel.setText("Thu");
				} else if (dayInt == Calendar.FRIDAY) {
					weekDayLabel.setText("Fri");
				} else if (dayInt == Calendar.SATURDAY) {
					weekDayLabel.setText("Sat");
				}
				this.weekDayLabelList.add(weekDayLabel);
			}

			return (JLabel) this.weekDayLabelList.elementAt(day);
		}

		private JLabel getMonthDayLabel(int dayOfMonth) {
			if (this.monthDayLabelList == null) {
				this.monthDayLabelList = new Vector(MAX_DAYS_IN_MONTH);
			}

			while (this.monthDayLabelList.size() < dayOfMonth + 1) {
				final int dayInt = this.monthDayLabelList.size() + 1;
				final JLabel monthDayLabel = new JLabel(String.valueOf(dayInt));
				monthDayLabel.setHorizontalAlignment(JLabel.CENTER);
				monthDayLabel.setForeground(this.foreground);
				monthDayLabel.addMouseListener(new MouseListener() {
					public void mousePressed(MouseEvent e) {
					}

					public void mouseClicked(MouseEvent e) {
					}

					public void mouseReleased(MouseEvent e) {
						monthDayLabel.setOpaque(false);
						monthDayLabel.setBackground(DatePopup.this.background);
						monthDayLabel.setForeground(DatePopup.this.foreground);
						DatePopup.this.calendar.set(Calendar.DAY_OF_MONTH, dayInt);
						
						Calendar newValue = DatePopup.this.calendar;
						//TDateComboBox.this.setCalendar(newValue);
						
						DatePopup.this.comboBox.requestFocus();
						DatePopup.this.togglePopup();
					}

					public void mouseEntered(MouseEvent e) {
						monthDayLabel.setOpaque(true);
						monthDayLabel.setBackground(DatePopup.this.selectedBackground);
						monthDayLabel.setForeground(DatePopup.this.selectedForeground);
					}

					public void mouseExited(MouseEvent e) {
						monthDayLabel.setOpaque(false);
						monthDayLabel.setBackground(DatePopup.this.background);
						monthDayLabel.setForeground(DatePopup.this.foreground);
					}
				});

				this.monthDayLabelList.add(monthDayLabel);
			}

			return (JLabel) this.monthDayLabelList.elementAt(dayOfMonth);
		}

		// update the Popup when either the month or the year of the calendar has been changed
		private void updatePopup() {
			this.getSelectedMonthLabel().setText(
					this.monthFormat.format(this.calendar.getTime()));
			this.getMonthPopupMenu().remove(getDaysPanel());
			this.updateDaysPanel();
			this.getMonthPopupMenu().add(BorderLayout.CENTER, getDaysPanel());
			this.getMonthPopupMenu().pack();
		}
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		//
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		//
	}
	
	public void setEnabled(boolean enable) {
		//
	}
	
	public boolean isEnabled() {
		return this.combo.isEnabled();
	}
}

